%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file is a simple simulation program to test control of an HVAC
% ideal air loads system
%
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialize model variables
delTim    = 60;  % [s] time step
TasIni    = 30;  % initial supply air temperature (in degrees C)
WasIni    = 0.4; % initial supply air humidity ratio
msIni     = 0;
output    = [TasIni WasIni msIni];
input     = [0,0];%input from E+[0 0 0]; u=[Tz Wz]
% Initialize flags
retVal    = 0;
flaWri    = 0;
flaRea    = 0;
simTimWri = 0;
simTimRea = 0;
Tas  = TasIni;
ms = msIni;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Add path to BCVTB matlab libraries
addpath('C:\Users\zbz\Documents\bcvtb\lib\matlab');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Establish the socket connection
sockfd = establishClientSocket('socket.cfg');
if sockfd < 0
  fprintf('Error: Failed to obtain socket file descriptor. sockfd=%d.\n', ...
          sockfd);
  exit;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Loop for simulation time steps.
simulate=true;
fprintf('Sim Loop')
while (simulate)
  % Assign values to be exchanged.
  try
    [retVal, flaRea, simTimRea, input ] = ...
        exchangeDoublesWithSocket(sockfd, flaWri, length(input), simTimWri, output);
  catch ME1
    % exchangeDoublesWithSocket had an error. Terminate the connection
    processError(ME1, sockfd, -1);
    simulate=false;
  end
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
  % Check return flags
  if (flaRea == 1) % End of simulation
    disp('Matlab received end of simulation flag from BCVTB. Exit simulation.');
    closeIPC(sockfd);
    simulate=false;
  end

  if (retVal < 0) % Error during data exchange
    exception = MException('BCVTB:RuntimeError', ...
                           'exchangeDoublesWithSocket returned value %d', ...
                           retVal);
    processError(exception, sockfd, -1);
    simulate=false;
  end

  if (flaRea > 1) % BCVTB requests termination due to an error.
    exception = MException('BCVTB:RuntimeError', ...
                           ['BCVTB requested MATLAB to terminate by sending %d\n', ...
                            'Exit simulation.\n'], retVal);
    processError(exception, sockfd, -1);
    simulate=false;
  end
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
  % No flags have been found that require termination of the 
  % simulation. 
  % Having obtained u_k, we compute the new state x_k+1 = f(u_k)
  % This is the actual simulation of the client.
  if (simulate)
      if input(1)> 25
          ms=1000; 
          Tas=15;
      elseif input(1)<15
          ms=1000; 
          Tas=30;
      else
          ms=0;
      end
    output=[Tas WasIni ms];
    % Advance simulation time
    simTimWri = simTimWri + delTim;
  end
end
exit
